import type { TextDocument } from '@volar/language-service';
import type { Node as FlatNode } from 'EmmetFlatNode';
export declare function getRootNode(document: TextDocument, useCache: boolean): FlatNode;
//# sourceMappingURL=parseDocument.d.ts.map