import type { Code } from '../../types';
export declare function generateIntersectMerge(generates: Iterable<Code>[]): Generator<Code>;
export declare function generateSpreadMerge(generates: Iterable<Code>[]): Generator<Code>;
