import { type Language, type SourceScript, type VueVirtualCode } from '@vue/language-core';
import type * as ts from 'typescript';
interface ExtractPropsInfo {
    name: string;
    type: string;
    model: boolean;
}
export declare function collectExtractProps(ts: typeof import('typescript'), language: Language, program: ts.Program, sourceScript: SourceScript, virtualCode: VueVirtualCode, templateCodeRange: [number, number], leadingOffset?: number): ExtractPropsInfo[];
export {};
