export * from '@volar/language-core';
export { mergeWorkspaceEdits } from './lib/features/provideRenameEdits';
export * from './lib/languageService';
export * from './lib/types';
export * from './lib/utils/transform';
export * from './lib/utils/uriMap';
