import type * as CompilerDOM from '@vue/compiler-dom';
export interface InlayHintInfo {
    blockName: string;
    offset: number;
    setting: string;
    label: string;
    tooltip?: string;
    paddingRight?: boolean;
    paddingLeft?: boolean;
}
export declare function createVBindShorthandInlayHintInfo(loc: CompilerDOM.SourceLocation, variableName: string): InlayHintInfo;
