declare const raw: {
    base: string;
    export: string;
    components: string;
    componentsOption: string;
    ctx: string;
    defaultModels: string;
    defaults: string;
    directives: string;
    directivesOption: string;
    dollars: string;
    emit: string;
    exposed: string;
    intrinsics: string;
    modelEmit: string;
    props: string;
    propsOption: string;
    self: string;
    setup: string;
    slots: string;
    src: string;
    Emit: string;
    EmitProps: string;
    GlobalComponents: string;
    InheritedAttrs: string;
    LocalComponents: string;
    LocalDirectives: string;
    ModelEmit: string;
    ModelProps: string;
    Props: string;
    PublicProps: string;
    RootEl: string;
    SetupExposed: string;
    Slots: string;
    StyleModules: string;
    StyleScopedClasses: string;
    TemplateRefs: string;
    asFunctionalComponent0: string;
    asFunctionalComponent1: string;
    asFunctionalDirective: string;
    asFunctionalElement0: string;
    asFunctionalElement1: string;
    asFunctionalSlot: string;
    directiveBindingRestFields: string;
    functionalComponentArgsRest: string;
    tryAsConstant: string;
    vFor: string;
    vSlot: string;
    ConstructorOverloads: string;
    Elements: string;
    EmitsToProps: string;
    FunctionalComponent0: string;
    FunctionalComponent1: string;
    FunctionalComponentCtx: string;
    FunctionalComponentProps: string;
    IsAny: string;
    IsFunction: string;
    NormalizeComponentEvent: string;
    NormalizeEmits: string;
    OverloadUnion: string;
    OverloadUnionInner: string;
    PickNotAny: string;
    PrettifyGlobal: string;
    ResolveDirectives: string;
    ResolveEmits: string;
    ShortEmits: string;
    ShortEmitsToObject: string;
    SpreadMerge: string;
    UnionToIntersection: string;
    WithComponent: string;
};
export declare const names: typeof raw;
export {};
