"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.names = void 0;
const raw = {
    base: '',
    export: '',
    components: '',
    componentsOption: '',
    ctx: '',
    defaultModels: '',
    defaults: '',
    directives: '',
    directivesOption: '',
    dollars: '',
    emit: '',
    exposed: '',
    intrinsics: '',
    modelEmit: '',
    props: '',
    propsOption: '',
    self: '',
    setup: '',
    slots: '',
    src: '',
    Emit: '',
    EmitProps: '',
    GlobalComponents: '',
    InheritedAttrs: '',
    LocalComponents: '',
    LocalDirectives: '',
    ModelEmit: '',
    ModelProps: '',
    Props: '',
    PublicProps: '',
    RootEl: '',
    SetupExposed: '',
    Slots: '',
    StyleModules: '',
    StyleScopedClasses: '',
    TemplateRefs: '',
    // #region template helpers
    asFunctionalComponent0: '',
    asFunctionalComponent1: '',
    asFunctionalDirective: '',
    asFunctionalElement0: '',
    asFunctionalElement1: '',
    asFunctionalSlot: '',
    directiveBindingRestFields: '',
    functionalComponentArgsRest: '',
    tryAsConstant: '',
    vFor: '',
    vSlot: '',
    ConstructorOverloads: '',
    Elements: '',
    EmitsToProps: '',
    FunctionalComponent0: '',
    FunctionalComponent1: '',
    FunctionalComponentCtx: '',
    FunctionalComponentProps: '',
    IsAny: '',
    IsFunction: '',
    NormalizeComponentEvent: '',
    NormalizeEmits: '',
    OverloadUnion: '',
    OverloadUnionInner: '',
    PickNotAny: '',
    PrettifyGlobal: '',
    ResolveDirectives: '',
    ResolveEmits: '',
    ShortEmits: '',
    ShortEmitsToObject: '',
    SpreadMerge: '',
    UnionToIntersection: '',
    WithComponent: '',
    // #endregion
};
exports.names = Object.fromEntries(Object.keys(raw).map(key => [key, `__VLS_${key}`]));
//# sourceMappingURL=names.js.map